@@ -2,15 +2,12 @@
 Special Optimized Methods - Integration of Special Methods
 
 This module provides optimized versions of existing fractional calculus methods
-that integrate the new special methods (Fractional Laplacian, Fractional Fourier Transform, 
+that integrate the new special methods (Fractional Laplacian, Fractional Fourier Transform,
 Fractional Z-Transform) for improved performance.
 """
 
 import numpy as np
-from numba import jit as numba_jit, prange
-from typing import Union, Optional, Callable, Tuple
-import warnings
-import time
+from typing import Union, Optional, Callable
 
 from ..core.definitions import FractionalOrder
 from .special_methods import (
@@ -23,32 +20,32 @@ from .special_methods import (
 class SpecialOptimizedWeylDerivative:
     """
     Weyl derivative optimized using Fractional Fourier Transform.
-    
+
     This implementation replaces the standard FFT convolution approach
     with Fractional Fourier Transform for better performance, especially
     for large arrays and specific alpha values.
     """
-    
+
     def __init__(self, alpha: Union[float, FractionalOrder]):
         """Initialize special optimized Weyl derivative calculator."""
         if isinstance(alpha, (int, float)):
             self.alpha = FractionalOrder(alpha)
         else:
             self.alpha = alpha
-            
+
         self.alpha_val = self.alpha.alpha
-        
+
         # Initialize special methods
         self.frft = FractionalFourierTransform(alpha)
-        
+
         # Determine optimal method based on alpha
         self._determine_optimal_method()
-    
+
     def _determine_optimal_method(self):
         """Determine the optimal computation method based on alpha value."""
         # For now, use standard FFT as it's more reliable
         self.optimal_method = "standard_fft"
-    
+
     def compute(
         self,
         f: Union[Callable, np.ndarray],
@@ -72,15 +69,15 @@ class SpecialOptimizedWeylDerivative:
                 x_array = x
             else:
                 x_array = np.arange(len(f)) * (h or 1.0)
-        
+
         # Ensure arrays have the same length
         min_len = min(len(f_array), len(x_array))
         f_array = f_array[:min_len]
         x_array = x_array[:min_len]
-        
+
         # Use specified method or optimal method
         method = method or self.optimal_method
-        
+
         if method == "fractional_fourier":
             return self._compute_fractional_fourier(f_array, x_array, h or 1.0)
         elif method == "standard_fft":
@@ -89,56 +86,68 @@ class SpecialOptimizedWeylDerivative:
             return self._compute_hybrid(f_array, x_array, h or 1.0)
         else:
             raise ValueError(f"Unknown method: {method}")
-    
-    def _compute_fractional_fourier(self, f: np.ndarray, x: np.ndarray, h: float) -> np.ndarray:
+
+    def _compute_fractional_fourier(
+            self,
+            f: np.ndarray,
+            x: np.ndarray,
+            h: float) -> np.ndarray:
         """Compute using Fractional Fourier Transform."""
-        N = len(f)
-        
+        len(f)
+
         # Apply FrFT
         u, f_frft = self.frft.transform(f, x, h, method="discrete")
-        
+
         # Apply Weyl kernel in FrFT domain
         # For Weyl derivative, we need to apply a specific kernel
         kernel = self._compute_weyl_kernel(u, h)
-        
+
         # Apply kernel in FrFT domain
         result_frft = f_frft * kernel
-        
+
         # For inverse FrFT, use 2π - alpha to avoid negative values
         inv_alpha = 2 * np.pi - self.alpha_val
         frft_inv = FractionalFourierTransform(inv_alpha)
         _, result = frft_inv.transform(result_frft, u, h, method="discrete")
-        
+
         return np.real(result)
-    
-    def _compute_standard_fft(self, f: np.ndarray, x: np.ndarray, h: float) -> np.ndarray:
+
+    def _compute_standard_fft(
+            self,
+            f: np.ndarray,
+            x: np.ndarray,
+            h: float) -> np.ndarray:
         """Compute using standard FFT (for alpha close to π/2)."""
         N = len(f)
-        
+
         # Standard FFT approach
         f_fft = np.fft.fft(f)
         freq = np.fft.fftfreq(N, h)
-        
+
         # Weyl derivative kernel in frequency domain
         kernel = (1j * 2 * np.pi * freq) ** self.alpha_val
         kernel[0] = 0  # Handle zero frequency
-        
+
         # Apply kernel
         result_fft = f_fft * kernel
-        
+
         # Inverse FFT
         result = np.real(np.fft.ifft(result_fft))
-        
+
         return result
-    
-    def _compute_hybrid(self, f: np.ndarray, x: np.ndarray, h: float) -> np.ndarray:
+
+    def _compute_hybrid(
+            self,
+            f: np.ndarray,
+            x: np.ndarray,
+            h: float) -> np.ndarray:
         """Hybrid approach using both methods."""
         # For large arrays, use FrFT
         if len(f) > 1000:
             return self._compute_fractional_fourier(f, x, h)
         else:
             return self._compute_standard_fft(f, x, h)
-    
+
     def _compute_weyl_kernel(self, u: np.ndarray, h: float) -> np.ndarray:
         """Compute Weyl derivative kernel in FrFT domain."""
         # Simplified Weyl kernel for FrFT domain
@@ -149,23 +158,23 @@ class SpecialOptimizedWeylDerivative:
 class SpecialOptimizedMarchaudDerivative:
     """
     Marchaud derivative optimized using Fractional Z-Transform.
-    
+
     This implementation replaces the difference quotient convolution
     with Fractional Z-Transform for better performance on discrete signals.
     """
-    
+
     def __init__(self, alpha: Union[float, FractionalOrder]):
         """Initialize special optimized Marchaud derivative calculator."""
         if isinstance(alpha, (int, float)):
             self.alpha = FractionalOrder(alpha)
         else:
             self.alpha = alpha
-            
+
         self.alpha_val = self.alpha.alpha
-        
+
         # Initialize special methods
         self.z_transform = FractionalZTransform(alpha)
-    
+
     def compute(
         self,
         f: Union[Callable, np.ndarray],
@@ -189,50 +198,58 @@ class SpecialOptimizedMarchaudDerivative:
                 x_array = x
             else:
                 x_array = np.arange(len(f)) * (h or 1.0)
-        
+
         # Ensure arrays have the same length
         min_len = min(len(f_array), len(x_array))
         f_array = f_array[:min_len]
         x_array = x_array[:min_len]
-        
+
         if method == "z_transform":
             return self._compute_z_transform(f_array, x_array, h or 1.0)
         elif method == "standard":
             return self._compute_standard(f_array, x_array, h or 1.0)
         else:
             raise ValueError(f"Unknown method: {method}")
-    
-    def _compute_z_transform(self, f: np.ndarray, x: np.ndarray, h: float) -> np.ndarray:
+
+    def _compute_z_transform(
+            self,
+            f: np.ndarray,
+            x: np.ndarray,
+            h: float) -> np.ndarray:
         """Compute using Fractional Z-Transform."""
         N = len(f)
-        
+
         # Create unit circle evaluation points
-        theta = np.linspace(0, 2*np.pi, N, endpoint=False)
+        theta = np.linspace(0, 2 * np.pi, N, endpoint=False)
         z_unit = np.exp(1j * theta)
-        
+
         # Apply Z-transform
         F_z = self.z_transform.transform(f, z_unit, method="fft")
-        
+
         # Apply Marchaud derivative operator in Z-domain
         # Marchaud derivative corresponds to multiplication by (1-z^(-1))^α
         marchaud_operator = (1 - z_unit**(-1)) ** self.alpha_val
-        
+
         # Apply operator
         result_z = F_z * marchaud_operator
-        
+
         # Inverse Z-transform (simplified)
         result = np.real(np.fft.ifft(result_z))
-        
+
         return result
-    
-    def _compute_standard(self, f: np.ndarray, x: np.ndarray, h: float) -> np.ndarray:
+
+    def _compute_standard(
+            self,
+            f: np.ndarray,
+            x: np.ndarray,
+            h: float) -> np.ndarray:
         """Standard Marchaud derivative computation."""
         N = len(f)
         result = np.zeros(N)
-        
+
         # Marchaud coefficient
         coeff = self.alpha_val / (np.pi * np.sin(self.alpha_val * np.pi))
-        
+
         for i in range(1, N):
             integral = 0.0
             for j in range(1, min(i, 1000)):  # Limit integration range
@@ -240,32 +257,32 @@ class SpecialOptimizedMarchaudDerivative:
                 if i - j >= 0:
                     diff = f[i] - f[i - j]
                     integral += diff / (tau ** (self.alpha_val + 1))
-            
+
             result[i] = coeff * integral * h
-        
+
         return result
 
 
 class SpecialOptimizedReizFellerDerivative:
     """
     Reiz-Feller derivative optimized using Fractional Laplacian.
-    
+
     This implementation replaces the spectral method with Fractional Laplacian
     for better performance and numerical stability.
     """
-    
+
     def __init__(self, alpha: Union[float, FractionalOrder]):
         """Initialize special optimized Reiz-Feller derivative calculator."""
         if isinstance(alpha, (int, float)):
             self.alpha = FractionalOrder(alpha)
         else:
             self.alpha = alpha
-            
+
         self.alpha_val = self.alpha.alpha
-        
+
         # Initialize special methods
         self.laplacian = FractionalLaplacian(alpha)
-    
+
     def compute(
         self,
         f: Union[Callable, np.ndarray],
@@ -289,64 +306,73 @@ class SpecialOptimizedReizFellerDerivative:
                 x_array = x
             else:
                 x_array = np.arange(len(f)) * (h or 1.0)
-        
+
         # Ensure arrays have the same length
         min_len = min(len(f_array), len(x_array))
         f_array = f_array[:min_len]
         x_array = x_array[:min_len]
-        
+
         if method == "laplacian":
             return self._compute_laplacian(f_array, x_array, h or 1.0)
         elif method == "spectral":
             return self._compute_spectral(f_array, x_array, h or 1.0)
         else:
             raise ValueError(f"Unknown method: {method}")
-    
-    def _compute_laplacian(self, f: np.ndarray, x: np.ndarray, h: float) -> np.ndarray:
+
+    def _compute_laplacian(
+            self,
+            f: np.ndarray,
+            x: np.ndarray,
+            h: float) -> np.ndarray:
         """Compute using Fractional Laplacian."""
         # Reiz-Feller derivative is related to fractional Laplacian
         # For symmetric functions, they are equivalent up to a constant
         result = self.laplacian.compute(f, x, h, method="spectral")
-        
+
         # Apply Reiz-Feller specific scaling
-        scaling_factor = 2 ** (self.alpha_val - 1) * np.sin(self.alpha_val * np.pi / 2)
+        scaling_factor = 2 ** (self.alpha_val - 1) * \
+            np.sin(self.alpha_val * np.pi / 2)
         return result * scaling_factor
-    
-    def _compute_spectral(self, f: np.ndarray, x: np.ndarray, h: float) -> np.ndarray:
+
+    def _compute_spectral(
+            self,
+            f: np.ndarray,
+            x: np.ndarray,
+            h: float) -> np.ndarray:
         """Standard spectral method computation."""
         N = len(f)
-        
+
         # Ensure N is even for FFT
         if N % 2 == 1:
             N += 1
             f = np.pad(f, (0, 1), mode="edge")
             x = np.pad(x, (0, 1), mode="edge")
-        
+
         # Compute FFT
         f_fft = np.fft.fft(f)
         freq = np.fft.fftfreq(N, h)
-        
+
         # Apply spectral filter |ξ|^α
         spectral_filter = np.abs(freq) ** self.alpha_val
         spectral_filter[0] = 0  # Handle zero frequency
-        
+
         # Apply filter in frequency domain
         filtered_fft = f_fft * spectral_filter
-        
+
         # Inverse FFT
         result = np.real(np.fft.ifft(filtered_fft))
-        
+
         return result[:len(f)]  # Return original length
 
 
 class UnifiedSpecialMethods:
     """
     Unified interface for all special methods with automatic method selection.
-    
+
     This class provides a unified API that automatically selects the best
     special method based on the problem characteristics.
     """
-    
+
     def __init__(self):
         """Initialize unified special methods interface."""
         self.methods = {
@@ -354,7 +380,7 @@ class UnifiedSpecialMethods:
             'fourier': FractionalFourierTransform,
             'z_transform': FractionalZTransform,
         }
-    
+
     def compute_derivative(
         self,
         f: Union[Callable, np.ndarray],
@@ -366,7 +392,7 @@ class UnifiedSpecialMethods:
     ) -> np.ndarray:
         """
         Compute fractional derivative using optimal special method.
-        
+
         Args:
             f: Function or function values
             x: Domain points
@@ -374,7 +400,7 @@ class UnifiedSpecialMethods:
             h: Step size
             method: Specific method to use (if None, auto-select)
             problem_type: Type of problem ("periodic", "discrete", "spectral", "general")
-            
+
         Returns:
             Derivative values
         """
@@ -383,10 +409,11 @@ class UnifiedSpecialMethods:
             f_array = np.array([f(xi) for xi in x])
         else:
             f_array = f
-            
+
         if method is None:
-            method = self._auto_select_method(problem_type, len(f_array), alpha)
-        
+            method = self._auto_select_method(
+                problem_type, len(f_array), alpha)
+
         if method == "laplacian":
             laplacian = FractionalLaplacian(alpha)
             return laplacian.compute(f_array, x, h, method="spectral")
@@ -396,19 +423,24 @@ class UnifiedSpecialMethods:
             return np.real(result)
         elif method == "z_transform":
             z_transform = FractionalZTransform(alpha)
-            z_values = np.exp(1j * np.linspace(0, 2*np.pi, len(f_array), endpoint=False))
+            z_values = np.exp(1j * np.linspace(0, 2 * np.pi,
+                              len(f_array), endpoint=False))
             result = z_transform.transform(f_array, z_values, method="fft")
             return np.real(np.fft.ifft(result))
         else:
             raise ValueError(f"Unknown method: {method}")
-    
-    def _auto_select_method(self, problem_type: str, size: int, alpha: Union[float, FractionalOrder]) -> str:
+
+    def _auto_select_method(self,
+                            problem_type: str,
+                            size: int,
+                            alpha: Union[float,
+                                         FractionalOrder]) -> str:
         """Automatically select the best method based on problem characteristics."""
         if isinstance(alpha, FractionalOrder):
             alpha_val = alpha.alpha
         else:
             alpha_val = alpha
-        
+
         # Method selection logic
         if problem_type == "periodic":
             return "fourier"
@@ -419,7 +451,7 @@ class UnifiedSpecialMethods:
         elif size > 1000:
             # For large arrays, prefer Laplacian (fastest)
             return "laplacian"
-        elif abs(alpha_val - np.pi/2) < 0.1:
+        elif abs(alpha_val - np.pi / 2) < 0.1:
             # For alpha close to π/2, use Fourier
             return "fourier"
         else:
