@@ -9,8 +9,7 @@ for training and inference.
 import torch
 import torch.nn as nn
 import torch.nn.functional as F
-from typing import Optional, Tuple, Union, List, Callable
-import numpy as np
+from typing import Tuple, List
 from dataclasses import dataclass
 
 from ..core.definitions import FractionalOrder
@@ -31,28 +30,29 @@ class AdjointConfig:
 class AdjointFractionalDerivative(torch.autograd.Function):
     """
     Adjoint-optimized fractional derivative using automatic differentiation
-    
+
     This implementation uses PyTorch's automatic differentiation with
     memory-efficient adjoint methods for optimal performance.
     """
-    
+
     @staticmethod
-    def forward(ctx, x: torch.Tensor, alpha: float, method: str = "RL") -> torch.Tensor:
+    def forward(ctx, x: torch.Tensor, alpha: float,
+                method: str = "RL") -> torch.Tensor:
         """
         Forward pass with adjoint optimization
-        
+
         Args:
             x: Input tensor
             alpha: Fractional order
             method: Derivative method ("RL", "Caputo", "GL")
-            
+
         Returns:
             Fractional derivative tensor
         """
         ctx.save_for_backward(x)
         ctx.alpha = alpha
         ctx.method = method
-        
+
         # Apply fractional derivative with adjoint optimization
         if method == "RL":
             return _adjoint_riemann_liouville_forward(x, alpha)
@@ -62,147 +62,157 @@ class AdjointFractionalDerivative(torch.autograd.Function):
             return _adjoint_grunwald_letnikov_forward(x, alpha)
         else:
             raise ValueError(f"Unknown method: {method}")
-    
+
     @staticmethod
-    def backward(ctx, grad_output: torch.Tensor) -> Tuple[torch.Tensor, None, None]:
+    def backward(
+            ctx, grad_output: torch.Tensor) -> Tuple[torch.Tensor, None, None]:
         """
         Backward pass using adjoint method for optimal memory usage
-        
+
         Args:
             grad_output: Gradient of the output
-            
+
         Returns:
             Gradient with respect to input, None for alpha, None for method
         """
         x, = ctx.saved_tensors
         alpha = ctx.alpha
         method = ctx.method
-        
+
         # Use adjoint method for gradient computation
         if method == "RL":
-            grad_input = _adjoint_riemann_liouville_backward(grad_output, x, alpha)
+            grad_input = _adjoint_riemann_liouville_backward(
+                grad_output, x, alpha)
         elif method == "Caputo":
             grad_input = _adjoint_caputo_backward(grad_output, x, alpha)
         elif method == "GL":
-            grad_input = _adjoint_grunwald_letnikov_backward(grad_output, x, alpha)
+            grad_input = _adjoint_grunwald_letnikov_backward(
+                grad_output, x, alpha)
         else:
             grad_input = grad_output
-        
+
         return grad_input, None, None
 
 
-def _adjoint_riemann_liouville_forward(x: torch.Tensor, alpha: float) -> torch.Tensor:
+def _adjoint_riemann_liouville_forward(
+        x: torch.Tensor,
+        alpha: float) -> torch.Tensor:
     """
     Adjoint-optimized Riemann-Liouville fractional derivative forward pass
-    
+
     Uses memory-efficient computation with optimal memory layout.
     """
     if alpha == 0:
         return x
-    
+
     if alpha == 1:
         return torch.gradient(x, dim=-1)[0]
-    
+
     # For non-integer alpha, use adjoint-optimized approximation
     result = x.clone()
-    
+
     if alpha > 0 and alpha < 1:
         # Use adjoint method: compute gradient once and reuse
         gradient = torch.gradient(x, dim=-1)[0]
-        
+
         # Apply fractional derivative with optimal memory usage
         result = (1 - alpha) * x + alpha * gradient
-        
+
         # Memory optimization: clear intermediate tensors
         del gradient
         torch.cuda.empty_cache() if x.is_cuda else None
-        
+
     elif alpha > 1:
         # For alpha > 1, use iterative adjoint method
         n = int(alpha)
         fractional_part = alpha - n
-        
+
         # Iterative computation with memory optimization
         current = x
         for i in range(n):
             current = torch.gradient(current, dim=-1)[0]
-            
+
             # Checkpoint for memory efficiency
             if i % 10 == 0 and current.is_cuda:
                 torch.cuda.empty_cache()
-        
+
         if fractional_part > 0:
             gradient = torch.gradient(current, dim=-1)[0]
-            result = (1 - fractional_part) * current + fractional_part * gradient
+            result = (1 - fractional_part) * current + \
+                fractional_part * gradient
             del gradient, current
         else:
             result = current
             del current
-        
+
         # Final memory cleanup
         torch.cuda.empty_cache() if x.is_cuda else None
-    
+
     return result
 
 
-def _adjoint_riemann_liouville_backward(grad_output: torch.Tensor, x: torch.Tensor, alpha: float) -> torch.Tensor:
+def _adjoint_riemann_liouville_backward(
+        grad_output: torch.Tensor,
+        x: torch.Tensor,
+        alpha: float) -> torch.Tensor:
     """
     Adjoint-optimized Riemann-Liouville fractional derivative backward pass
-    
+
     Uses adjoint method for optimal memory usage and computational efficiency.
     """
     if alpha == 0:
         return grad_output
-    
+
     if alpha == 1:
         # For first derivative, the adjoint is -gradient
         return -torch.gradient(grad_output, dim=-1)[0]
-    
+
     # Adjoint method for fractional derivatives
     if alpha > 0 and alpha < 1:
         # Compute adjoint efficiently
         gradient_grad = torch.gradient(grad_output, dim=-1)[0]
         result = (1 - alpha) * grad_output - alpha * gradient_grad
-        
+
         # Memory optimization
         del gradient_grad
         torch.cuda.empty_cache() if grad_output.is_cuda else None
-        
+
         return result
-        
+
     elif alpha > 1:
         # Iterative adjoint computation
         n = int(alpha)
         fractional_part = alpha - n
-        
+
         # Backward adjoint iteration
         current_grad = grad_output
         for i in range(n):
             current_grad = -torch.gradient(current_grad, dim=-1)[0]
-            
+
             # Memory optimization
             if i % 10 == 0 and current_grad.is_cuda:
                 torch.cuda.empty_cache()
-        
+
         if fractional_part > 0:
             gradient_grad = torch.gradient(current_grad, dim=-1)[0]
-            result = (1 - fractional_part) * current_grad - fractional_part * gradient_grad
+            result = (1 - fractional_part) * current_grad - \
+                fractional_part * gradient_grad
             del gradient_grad, current_grad
         else:
             result = current_grad
             del current_grad
-        
+
         # Final cleanup
         torch.cuda.empty_cache() if grad_output.is_cuda else None
         return result
-    
+
     return grad_output
 
 
 def _adjoint_caputo_forward(x: torch.Tensor, alpha: float) -> torch.Tensor:
     """
     Adjoint-optimized Caputo fractional derivative forward pass
-    
+
     Similar to Riemann-Liouville but with Caputo-specific optimizations.
     """
     # For this implementation, Caputo is similar to Riemann-Liouville
@@ -210,67 +220,76 @@ def _adjoint_caputo_forward(x: torch.Tensor, alpha: float) -> torch.Tensor:
     return _adjoint_riemann_liouville_forward(x, alpha)
 
 
-def _adjoint_caputo_backward(grad_output: torch.Tensor, x: torch.Tensor, alpha: float) -> torch.Tensor:
+def _adjoint_caputo_backward(
+        grad_output: torch.Tensor,
+        x: torch.Tensor,
+        alpha: float) -> torch.Tensor:
     """
     Adjoint-optimized Caputo fractional derivative backward pass
     """
     return _adjoint_riemann_liouville_backward(grad_output, x, alpha)
 
 
-def _adjoint_grunwald_letnikov_forward(x: torch.Tensor, alpha: float) -> torch.Tensor:
+def _adjoint_grunwald_letnikov_forward(
+        x: torch.Tensor,
+        alpha: float) -> torch.Tensor:
     """
     Adjoint-optimized Grünwald-Letnikov fractional derivative forward pass
-    
+
     Uses efficient finite difference approximation with adjoint optimization.
     """
     if alpha == 0:
         return x
-    
+
     if alpha == 1:
         return torch.gradient(x, dim=-1)[0]
-    
+
     # Efficient GL approximation with adjoint method
     result = x.clone()
-    
+
     if alpha > 0 and alpha < 1:
         # Use weighted finite differences with optimal memory usage
         gradient = torch.gradient(x, dim=-1)[0]
-        
+
         # Apply fractional derivative efficiently
         result = (1 - alpha) * x + alpha * gradient
-        
+
         # Memory optimization
         del gradient
         torch.cuda.empty_cache() if x.is_cuda else None
-        
+
     elif alpha > 1:
         # Iterative GL with memory optimization
         n = int(alpha)
         fractional_part = alpha - n
-        
+
         current = x
         for i in range(n):
             current = torch.gradient(current, dim=-1)[0]
-            
+
             # Memory checkpointing
             if i % 10 == 0 and current.is_cuda:
                 torch.cuda.empty_cache()
-        
+
         if fractional_part > 0:
             gradient = torch.gradient(current, dim=-1)[0]
-            result = (1 - fractional_part) * current + fractional_part * gradient
+            result = (1 - fractional_part) * current + \
+                fractional_part * gradient
             del gradient, current
         else:
             result = current
             del current
-        
+
         # Final cleanup
         torch.cuda.empty_cache() if x.is_cuda else None
-    
+
     return result
 
 
-def _adjoint_grunwald_letnikov_backward(grad_output: torch.Tensor, x: torch.Tensor, alpha: float) -> torch.Tensor:
+def _adjoint_grunwald_letnikov_backward(
+        grad_output: torch.Tensor,
+        x: torch.Tensor,
+        alpha: float) -> torch.Tensor:
     """
     Adjoint-optimized Grünwald-Letnikov fractional derivative backward pass
     """
@@ -280,26 +299,31 @@ def _adjoint_grunwald_letnikov_backward(grad_output: torch.Tensor, x: torch.Tens
 class AdjointFractionalLayer(nn.Module):
     """
     Adjoint-optimized fractional layer with memory efficiency
-    
+
     This layer uses adjoint methods for optimal memory usage and
     computational efficiency during training.
     """
-    
-    def __init__(self, alpha: float, method: str = "RL", config: AdjointConfig = None):
+
+    def __init__(
+            self,
+            alpha: float,
+            method: str = "RL",
+            config: AdjointConfig = None):
         super().__init__()
         self.alpha = FractionalOrder(alpha)
         self.method = method
         self.config = config or AdjointConfig()
-    
+
     def forward(self, x: torch.Tensor) -> torch.Tensor:
         """Forward pass with adjoint optimization"""
         if self.config.use_adjoint:
-            return AdjointFractionalDerivative.apply(x, self.alpha.alpha, self.method)
+            return AdjointFractionalDerivative.apply(
+                x, self.alpha.alpha, self.method)
         else:
             # Fallback to standard implementation
             from .fractional_autograd import fractional_derivative
             return fractional_derivative(x, self.alpha.alpha, self.method)
-    
+
     def extra_repr(self) -> str:
         return f'alpha={self.alpha}, method={self.method}, adjoint={self.config.use_adjoint}'
 
@@ -307,11 +331,11 @@ class AdjointFractionalLayer(nn.Module):
 class MemoryEfficientFractionalNetwork(nn.Module):
     """
     Memory-efficient fractional neural network using adjoint methods
-    
+
     This network uses checkpointing and adjoint methods to minimize
     memory usage during training while maintaining performance.
     """
-    
+
     def __init__(
         self,
         input_size: int,
@@ -323,7 +347,7 @@ class MemoryEfficientFractionalNetwork(nn.Module):
         adjoint_config: AdjointConfig = None
     ):
         super().__init__()
-        
+
         self.input_size = input_size
         self.hidden_sizes = hidden_sizes
         self.output_size = output_size
@@ -331,59 +355,65 @@ class MemoryEfficientFractionalNetwork(nn.Module):
         self.activation = activation
         self.dropout = dropout
         self.adjoint_config = adjoint_config or AdjointConfig()
-        
+
         # Build layers with adjoint optimization
         self.layers = nn.ModuleList()
-        
+
         # Input layer
         if hidden_sizes:
             self.layers.append(nn.Linear(input_size, hidden_sizes[0]))
-            
+
             # Hidden layers
             for i in range(len(hidden_sizes) - 1):
-                self.layers.append(nn.Linear(hidden_sizes[i], hidden_sizes[i + 1]))
-            
+                self.layers.append(
+                    nn.Linear(hidden_sizes[i], hidden_sizes[i + 1]))
+
             # Output layer
             self.layers.append(nn.Linear(hidden_sizes[-1], output_size))
         else:
             # Direct input to output
             self.layers.append(nn.Linear(input_size, output_size))
-        
+
         # Fractional derivative layer
         self.fractional_layer = AdjointFractionalLayer(
-            fractional_order, 
-            method="RL", 
+            fractional_order,
+            method="RL",
             config=self.adjoint_config
         )
-        
+
         # Dropout layer
         self.dropout_layer = nn.Dropout(dropout)
-    
-    def forward(self, x: torch.Tensor, use_fractional: bool = True, method: str = "RL") -> torch.Tensor:
+
+    def forward(
+            self,
+            x: torch.Tensor,
+            use_fractional: bool = True,
+            method: str = "RL") -> torch.Tensor:
         """
         Forward pass with memory-efficient adjoint methods
-        
+
         Args:
             x: Input tensor
             use_fractional: Whether to apply fractional derivatives
             method: Fractional derivative method
-            
+
         Returns:
             Output tensor
         """
         print(f"      Debug: Input to network: {x.shape}")
-        
+
         if use_fractional:
             # Apply fractional derivative with adjoint optimization
             x = self.fractional_layer(x)
             print(f"      Debug: After fractional layer: {x.shape}")
-        
+
         # Process through layers with memory optimization
         for i, layer in enumerate(self.layers[:-1]):
-            print(f"      Debug: Before layer {i} ({type(layer).__name__}): {x.shape}")
+            print(
+                f"      Debug: Before layer {i} ({type(layer).__name__}): {x.shape}")
             x = layer(x)
             print(f"      Debug: After layer {i}: {x.shape}")
-            
+
             # Apply activation
             if self.activation == "relu":
                 x = F.relu(x)
@@ -391,74 +421,80 @@ class MemoryEfficientFractionalNetwork(nn.Module):
                 x = torch.tanh(x)
             elif self.activation == "sigmoid":
                 x = torch.sigmoid(x)
-            
+
             # Apply dropout (except for last layer)
             if i < len(self.layers) - 2:
                 x = self.dropout_layer(x)
-            
+
             # Memory optimization: checkpointing for large networks
-            if (self.adjoint_config.memory_efficient and 
-                len(self.hidden_sizes) > 2 and 
-                i % self.adjoint_config.checkpoint_frequency == 0):
+            if (self.adjoint_config.memory_efficient and
+                len(self.hidden_sizes) > 2 and
+                    i % self.adjoint_config.checkpoint_frequency == 0):
                 # Only checkpoint the next few layers to maintain tensor shapes
-                checkpoint_end = min(i + 1 + self.adjoint_config.checkpoint_frequency, len(self.layers) - 1)
+                checkpoint_end = min(
+                    i + 1 + self.adjoint_config.checkpoint_frequency, len(self.layers) - 1)
                 x = torch.utils.checkpoint.checkpoint(
-                    lambda x: self._checkpoint_forward(x, i + 1, checkpoint_end),
+                    lambda x: self._checkpoint_forward(
+                        x, i + 1, checkpoint_end),
                     x,
                     preserve_rng_state=False
                 )
-        
+
         # Final layer
         print(f"      Debug: Before final layer: {x.shape}")
         x = self.layers[-1](x)
         print(f"      Debug: After final layer: {x.shape}")
         return x
-    
-    def _checkpoint_forward(self, x: torch.Tensor, start_layer: int, end_layer: int) -> torch.Tensor:
+
+    def _checkpoint_forward(
+            self,
+            x: torch.Tensor,
+            start_layer: int,
+            end_layer: int) -> torch.Tensor:
         """Checkpoint forward pass for memory efficiency"""
         for i in range(start_layer, end_layer):
             x = self.layers[i](x)
-            
+
             if self.activation == "relu":
                 x = F.relu(x)
             elif self.activation == "tanh":
                 x = torch.tanh(x)
             elif self.activation == "sigmoid":
                 x = torch.sigmoid(x)
-            
+
             if i < len(self.layers) - 2:
                 x = self.dropout_layer(x)
-        
+
         return x
 
 
 class AdjointOptimizer:
     """
     Adjoint-optimized optimizer for fractional neural networks
-    
+
     This optimizer uses adjoint methods to compute gradients more efficiently
     and with better memory usage.
     """
-    
+
     def __init__(self, model: nn.Module, config: AdjointConfig = None):
         self.model = model
         self.config = config or AdjointConfig()
         self.optimizer = torch.optim.Adam(model.parameters())
-        
+
         # Gradient accumulation for memory efficiency
         if self.config.gradient_accumulation:
             self.accumulation_steps = self.config.accumulation_steps
             self.accumulation_counter = 0
-    
+
     def step(self, loss: torch.Tensor):
         """Optimization step with adjoint optimization"""
         if self.config.gradient_accumulation:
             # Scale loss for gradient accumulation
             scaled_loss = loss / self.accumulation_steps
             scaled_loss.backward()
-            
+
             self.accumulation_counter += 1
-            
+
             if self.accumulation_counter % self.accumulation_steps == 0:
                 self.optimizer.step()
                 self.optimizer.zero_grad()
@@ -468,22 +504,25 @@ class AdjointOptimizer:
             loss.backward()
             self.optimizer.step()
             self.optimizer.zero_grad()
-    
+
     def zero_grad(self):
         """Zero gradients"""
         self.optimizer.zero_grad()
         self.accumulation_counter = 0
 
 
-def adjoint_fractional_derivative(x: torch.Tensor, alpha: float, method: str = "RL") -> torch.Tensor:
+def adjoint_fractional_derivative(
+        x: torch.Tensor,
+        alpha: float,
+        method: str = "RL") -> torch.Tensor:
     """
     Convenience function for adjoint-optimized fractional derivatives
-    
+
     Args:
         x: Input tensor
         alpha: Fractional order
         method: Derivative method
-        
+
     Returns:
         Fractional derivative tensor with adjoint optimization
     """
@@ -509,23 +548,23 @@ def adjoint_gl_derivative(x: torch.Tensor, alpha: float) -> torch.Tensor:
 if __name__ == "__main__":
     # Test the adjoint optimization
     print("🧠 Testing Adjoint Fractional Derivatives...")
-    
+
     # Create test data
     x = torch.randn(100, 10, requires_grad=True)
     alpha = 0.5
-    
+
     # Test adjoint derivatives
     rl_result = adjoint_rl_derivative(x, alpha)
     caputo_result = adjoint_caputo_derivative(x, alpha)
     gl_result = adjoint_gl_derivative(x, alpha)
-    
+
     print(f"✅ Adjoint RL: {rl_result.shape}")
     print(f"✅ Adjoint Caputo: {caputo_result.shape}")
     print(f"✅ Adjoint GL: {gl_result.shape}")
-    
+
     # Test gradient flow
     loss = rl_result.sum()
     loss.backward()
-    
+
     print(f"✅ Gradient flow: {x.grad is not None}")
     print("🎉 Adjoint optimization working correctly!")
