@@ -6,15 +6,13 @@ metadata, and deployment status to support the development vs. production workfl
 """
 
 import json
-import pickle
 import hashlib
 import sqlite3
-from datetime import datetime, timedelta
-from typing import Dict, List, Optional, Any, Union
+from datetime import datetime
+from typing import Dict, List, Optional, Any
 from dataclasses import dataclass, asdict
 from pathlib import Path
 import torch
-import numpy as np
 from enum import Enum
 
 
@@ -50,7 +48,7 @@ class ModelMetadata:
     checksum: str
     deployment_status: DeploymentStatus
     notes: str = ""
-    
+
     def to_dict(self) -> Dict[str, Any]:
         """Convert to dictionary for storage"""
         data = asdict(self)
@@ -58,7 +56,7 @@ class ModelMetadata:
         data['updated_at'] = self.updated_at.isoformat()
         data['deployment_status'] = self.deployment_status.value
         return data
-    
+
     @classmethod
     def from_dict(cls, data: Dict[str, Any]) -> 'ModelMetadata':
         """Create from dictionary"""
@@ -81,7 +79,7 @@ class ModelVersion:
     git_commit: str
     git_branch: str
     is_production: bool = False
-    
+
     def to_dict(self) -> Dict[str, Any]:
         """Convert to dictionary for storage"""
         data = asdict(self)
@@ -93,27 +91,30 @@ class ModelVersion:
 class ModelRegistry:
     """
     Central model registry for tracking and managing models
-    
+
     This class provides a comprehensive system for:
     - Storing model metadata and versions
     - Tracking deployment status
     - Managing development vs. production models
     - Version control and rollback capabilities
     """
-    
-    def __init__(self, db_path: str = "models/registry.db", storage_path: str = "models/"):
+
+    def __init__(
+            self,
+            db_path: str = "models/registry.db",
+            storage_path: str = "models/"):
         self.db_path = db_path
         self.storage_path = Path(storage_path)
         self.storage_path.mkdir(parents=True, exist_ok=True)
-        
+
         # Initialize database
         self._init_database()
-    
+
     def _init_database(self):
         """Initialize the SQLite database"""
         conn = sqlite3.connect(self.db_path)
         cursor = conn.cursor()
-        
+
         # Create tables
         cursor.execute('''
             CREATE TABLE IF NOT EXISTS models (
@@ -138,7 +139,7 @@ class ModelRegistry:
                 notes TEXT
             )
         ''')
-        
+
         cursor.execute('''
             CREATE TABLE IF NOT EXISTS versions (
                 version TEXT,
@@ -154,16 +155,16 @@ class ModelRegistry:
                 FOREIGN KEY (model_id) REFERENCES models (model_id)
             )
         ''')
-        
+
         conn.commit()
         conn.close()
-    
+
     def _generate_model_id(self, name: str, version: str) -> str:
         """Generate unique model ID"""
         timestamp = datetime.now().isoformat()
         unique_string = f"{name}_{version}_{timestamp}"
         return hashlib.md5(unique_string.encode()).hexdigest()[:12]
-    
+
     def _calculate_checksum(self, file_path: str) -> str:
         """Calculate SHA256 checksum of model file"""
         sha256_hash = hashlib.sha256()
@@ -171,7 +172,7 @@ class ModelRegistry:
             for chunk in iter(lambda: f.read(4096), b""):
                 sha256_hash.update(chunk)
         return sha256_hash.hexdigest()
-    
+
     def register_model(
         self,
         model: torch.nn.Module,
@@ -193,7 +194,7 @@ class ModelRegistry:
     ) -> str:
         """
         Register a new model in the registry
-        
+
         Args:
             model: PyTorch model to register
             name: Model name
@@ -211,23 +212,23 @@ class ModelRegistry:
             notes: Additional notes
             git_commit: Git commit hash
             git_branch: Git branch name
-            
+
         Returns:
             Model ID
         """
         model_id = self._generate_model_id(name, version)
-        
+
         # Create storage directories
         model_dir = self.storage_path / model_id
         model_dir.mkdir(exist_ok=True)
-        
+
         # Save model files
         model_path = model_dir / f"{name}_v{version}.pth"
         config_path = model_dir / f"{name}_v{version}_config.json"
-        
+
         # Save model state
         torch.save(model.state_dict(), model_path)
-        
+
         # Save configuration
         config_data = {
             'name': name,
@@ -239,11 +240,11 @@ class ModelRegistry:
         }
         with open(config_path, 'w') as f:
             json.dump(config_data, f, indent=2)
-        
+
         # Calculate file size and checksum
         file_size = model_path.stat().st_size
         checksum = self._calculate_checksum(str(model_path))
-        
+
         # Create metadata
         now = datetime.now()
         metadata = ModelMetadata(
@@ -267,7 +268,7 @@ class ModelRegistry:
             deployment_status=DeploymentStatus.DEVELOPMENT,
             notes=notes
         )
-        
+
         # Create version
         model_version = ModelVersion(
             version=version,
@@ -281,18 +282,18 @@ class ModelRegistry:
             git_branch=git_branch,
             is_production=False
         )
-        
+
         # Store in database
         self._store_model(metadata)
         self._store_version(model_version)
-        
+
         return model_id
-    
+
     def _store_model(self, metadata: ModelMetadata):
         """Store model metadata in database"""
         conn = sqlite3.connect(self.db_path)
         cursor = conn.cursor()
-        
+
         cursor.execute('''
             INSERT OR REPLACE INTO models VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)
         ''', (
@@ -316,15 +317,15 @@ class ModelRegistry:
             metadata.deployment_status.value,
             metadata.notes
         ))
-        
+
         conn.commit()
         conn.close()
-    
+
     def _store_version(self, model_version: ModelVersion):
         """Store model version in database"""
         conn = sqlite3.connect(self.db_path)
         cursor = conn.cursor()
-        
+
         cursor.execute('''
             INSERT OR REPLACE INTO versions VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?)
         ''', (
@@ -338,22 +339,22 @@ class ModelRegistry:
             model_version.git_branch,
             model_version.is_production
         ))
-        
+
         conn.commit()
         conn.close()
-    
+
     def get_model(self, model_id: str) -> Optional[ModelMetadata]:
         """Get model metadata by ID"""
         conn = sqlite3.connect(self.db_path)
         cursor = conn.cursor()
-        
+
         cursor.execute('SELECT * FROM models WHERE model_id = ?', (model_id,))
         row = cursor.fetchone()
         conn.close()
-        
+
         if row is None:
             return None
-        
+
         # Reconstruct metadata
         metadata = ModelMetadata(
             model_id=row[0],
@@ -376,18 +377,21 @@ class ModelRegistry:
             deployment_status=DeploymentStatus(row[17]),
             notes=row[18]
         )
-        
+
         return metadata
-    
+
     def get_model_versions(self, model_id: str) -> List[ModelVersion]:
         """Get all versions of a model"""
         conn = sqlite3.connect(self.db_path)
         cursor = conn.cursor()
-        
-        cursor.execute('SELECT * FROM versions WHERE model_id = ? ORDER BY created_at DESC', (model_id,))
+
+        cursor.execute(
+            'SELECT * FROM versions WHERE model_id = ? ORDER BY created_at DESC',
+            (model_id,
+             ))
         rows = cursor.fetchall()
         conn.close()
-        
+
         versions = []
         for row in rows:
             metadata = self.get_model(model_id)
@@ -405,55 +409,59 @@ class ModelRegistry:
                     is_production=bool(row[8])
                 )
                 versions.append(model_version)
-        
+
         return versions
-    
-    def update_deployment_status(self, model_id: str, version: str, status: DeploymentStatus):
+
+    def update_deployment_status(
+            self,
+            model_id: str,
+            version: str,
+            status: DeploymentStatus):
         """Update deployment status of a model version"""
         conn = sqlite3.connect(self.db_path)
         cursor = conn.cursor()
-        
+
         # Update deployment status
         cursor.execute('''
             UPDATE versions SET is_production = ? WHERE model_id = ? AND version = ?
         ''', (status == DeploymentStatus.PRODUCTION, model_id, version))
-        
+
         # Update model metadata
         cursor.execute('''
             UPDATE models SET deployment_status = ?, updated_at = ? WHERE model_id = ?
         ''', (status.value, datetime.now().isoformat(), model_id))
-        
+
         conn.commit()
         conn.close()
-    
+
     def promote_to_production(self, model_id: str, version: str):
         """Promote a model version to production"""
         # First, demote all other versions of this model
         conn = sqlite3.connect(self.db_path)
         cursor = conn.cursor()
-        
+
         cursor.execute('''
             UPDATE versions SET is_production = 0 WHERE model_id = ?
         ''', (model_id,))
-        
+
         # Then promote the specified version
         cursor.execute('''
             UPDATE versions SET is_production = 1 WHERE model_id = ? AND version = ?
         ''', (model_id, version))
-        
+
         # Update model status
         cursor.execute('''
             UPDATE models SET deployment_status = ?, updated_at = ? WHERE model_id = ?
         ''', (DeploymentStatus.PRODUCTION.value, datetime.now().isoformat(), model_id))
-        
+
         conn.commit()
         conn.close()
-    
+
     def get_production_models(self) -> List[ModelVersion]:
         """Get all production models"""
         conn = sqlite3.connect(self.db_path)
         cursor = conn.cursor()
-        
+
         cursor.execute('''
             SELECT v.*, m.name, m.description FROM versions v
             JOIN models m ON v.model_id = m.model_id
@@ -461,7 +469,7 @@ class ModelRegistry:
         ''')
         rows = cursor.fetchall()
         conn.close()
-        
+
         production_models = []
         for row in rows:
             metadata = self.get_model(row[1])
@@ -479,9 +487,9 @@ class ModelRegistry:
                     is_production=bool(row[8])
                 )
                 production_models.append(model_version)
-        
+
         return production_models
-    
+
     def search_models(
         self,
         name: Optional[str] = None,
@@ -493,35 +501,35 @@ class ModelRegistry:
         """Search for models based on criteria"""
         conn = sqlite3.connect(self.db_path)
         cursor = conn.cursor()
-        
+
         query = 'SELECT * FROM models WHERE 1=1'
         params = []
-        
+
         if name:
             query += ' AND name LIKE ?'
             params.append(f'%{name}%')
-        
+
         if tags:
             for tag in tags:
                 query += ' AND tags LIKE ?'
                 params.append(f'%{tag}%')
-        
+
         if model_type:
             query += ' AND model_type = ?'
             params.append(model_type)
-        
+
         if deployment_status:
             query += ' AND deployment_status = ?'
             params.append(deployment_status.value)
-        
+
         if author:
             query += ' AND author LIKE ?'
             params.append(f'%{author}%')
-        
+
         cursor.execute(query, params)
         rows = cursor.fetchall()
         conn.close()
-        
+
         models = []
         for row in rows:
             metadata = ModelMetadata(
@@ -546,83 +554,85 @@ class ModelRegistry:
                 notes=row[18]
             )
             models.append(metadata)
-        
+
         return models
-    
+
     def delete_model(self, model_id: str):
         """Delete a model and all its versions"""
         # Get model files
         versions = self.get_model_versions(model_id)
-        
+
         # Delete files
         for version in versions:
             if Path(version.model_path).exists():
                 Path(version.model_path).unlink()
             if Path(version.config_path).exists():
                 Path(version.config_path).unlink()
-        
+
         # Delete from database
         conn = sqlite3.connect(self.db_path)
         cursor = conn.cursor()
-        
+
         cursor.execute('DELETE FROM versions WHERE model_id = ?', (model_id,))
         cursor.execute('DELETE FROM models WHERE model_id = ?', (model_id,))
-        
+
         conn.commit()
         conn.close()
-    
+
     def export_registry(self, file_path: str):
         """Export registry to JSON file"""
         conn = sqlite3.connect(self.db_path)
         cursor = conn.cursor()
-        
+
         # Get all models
         cursor.execute('SELECT * FROM models')
         models = cursor.fetchall()
-        
+
         # Get all versions
         cursor.execute('SELECT * FROM versions')
         versions = cursor.fetchall()
-        
+
         conn.close()
-        
+
         # Export data
         export_data = {
             'exported_at': datetime.now().isoformat(),
             'models': models,
             'versions': versions
         }
-        
+
         with open(file_path, 'w') as f:
             json.dump(export_data, f, indent=2)
-    
+
     def get_registry_summary(self) -> Dict[str, Any]:
         """Get summary statistics of the registry"""
         conn = sqlite3.connect(self.db_path)
         cursor = conn.cursor()
-        
+
         # Total models
         cursor.execute('SELECT COUNT(*) FROM models')
         total_models = cursor.fetchone()[0]
-        
+
         # Total versions
         cursor.execute('SELECT COUNT(*) FROM versions')
         total_versions = cursor.fetchone()[0]
-        
+
         # Models by status
-        cursor.execute('SELECT deployment_status, COUNT(*) FROM models GROUP BY deployment_status')
+        cursor.execute(
+            'SELECT deployment_status, COUNT(*) FROM models GROUP BY deployment_status')
         status_counts = dict(cursor.fetchall())
-        
+
         # Models by type
-        cursor.execute('SELECT model_type, COUNT(*) FROM models GROUP BY model_type')
+        cursor.execute(
+            'SELECT model_type, COUNT(*) FROM models GROUP BY model_type')
         type_counts = dict(cursor.fetchall())
-        
+
         # Production models
         cursor.execute('SELECT COUNT(*) FROM versions WHERE is_production = 1')
         production_models = cursor.fetchone()[0]
-        
+
         conn.close()
-        
+
         return {
             'total_models': total_models,
             'total_versions': total_versions,
@@ -631,14 +641,15 @@ class ModelRegistry:
             'type_distribution': type_counts
         }
 
-    def reconstruct_model(self, model_id: str, version: str = None) -> Optional[torch.nn.Module]:
+    def reconstruct_model(self, model_id: str,
+                          version: str = None) -> Optional[torch.nn.Module]:
         """
         Reconstruct a model from its saved state dict and configuration
-        
+
         Args:
             model_id: Model ID
             version: Model version (if None, uses latest)
-            
+
         Returns:
             Reconstructed PyTorch model
         """
@@ -646,12 +657,12 @@ class ModelRegistry:
         model_metadata = self.get_model(model_id)
         if not model_metadata:
             return None
-        
+
         # Get model version
         model_versions = self.get_model_versions(model_id)
         if not model_versions:
             return None
-        
+
         target_version = None
         if version is None:
             target_version = model_versions[0]  # Latest version
@@ -660,29 +671,29 @@ class ModelRegistry:
                 if mv.version == version:
                     target_version = mv
                     break
-        
+
         if not target_version:
             return None
-        
+
         # Load configuration
         config_path = Path(target_version.config_path)
         if not config_path.exists():
             return None
-        
+
         with open(config_path, 'r') as f:
             config_data = json.load(f)
-        
+
         # Load state dict
         model_path = Path(target_version.model_path)
         if not model_path.exists():
             return None
-        
+
         state_dict = torch.load(model_path)
-        
+
         # Reconstruct model based on type
         model_type = config_data.get('model_type', 'FractionalNeuralNetwork')
         hyperparameters = config_data.get('hyperparameters', {})
-        
+
         if model_type == "fractional_neural_network":
             # Check if it's an adjoint-optimized network
             if "adjoint" in config_data.get('description', '').lower():
@@ -695,8 +706,10 @@ class ModelRegistry:
                 model = MemoryEfficientFractionalNetwork(
                     input_size=hyperparameters.get('input_size', 10),
                     hidden_sizes=hyperparameters.get('hidden_sizes', [64, 32]),
-                    output_size=hyperparameters.get('output_size', 1),  # Use saved output_size
-                    fractional_order=hyperparameters.get('fractional_order', 0.5),
+                    output_size=hyperparameters.get(
+                        'output_size', 1),  # Use saved output_size
+                    fractional_order=hyperparameters.get(
+                        'fractional_order', 0.5),
                     adjoint_config=adjoint_config
                 )
             else:
@@ -704,8 +717,10 @@ class ModelRegistry:
                 model = FractionalNeuralNetwork(
                     input_size=hyperparameters.get('input_size', 10),
                     hidden_sizes=hyperparameters.get('hidden_sizes', [64, 32]),
-                    output_size=hyperparameters.get('output_size', 1),  # Use saved output_size
-                    fractional_order=hyperparameters.get('fractional_order', 0.5)
+                    output_size=hyperparameters.get(
+                        'output_size', 1),  # Use saved output_size
+                    fractional_order=hyperparameters.get(
+                        'fractional_order', 0.5)
                 )
         else:
             # For other model types, create a simple placeholder
@@ -715,7 +730,7 @@ class ModelRegistry:
                 torch.nn.ReLU(),
                 torch.nn.Linear(64, hyperparameters.get('output_size', 1))
             )
-        
+
         # Load the state dict
         model.load_state_dict(state_dict)
         return model
